#include "map.hh"
#include "object.hh"
#include "xyz.hh"

Map::Map()
{
}

Map::~Map()
{
    objlist_t::iterator i;
    for(i=objs.begin(); i!=objs.end(); ++i)
        delete *i;
}

#if 0
#include <cstdio>
void Map::Load(const v3 *verts, unsigned vcount,
               const wall *walls, unsigned wcount)
{
    if(Walls) { delete[] Walls; Walls=NULL; }
    Walls = new MapWall[WallCount = wcount];
    FLOAT mix=0,miy=0,miz=0, max=0,may=0,maz=0;
    for(unsigned w=0; w<wcount; ++w)
    {
        if(walls[w].v1 >= vcount || walls[w].v2 >= vcount)
        {
            std::fprintf(stderr, "Wall %u has vertex out of range (max %u, has %u and %u)\n",
                w, vcount, walls[w].v1, walls[w].v2);
            continue;
        }
        Walls[w] = MapWall(walls[w], verts);
        if(!w || Walls[w].minx()<mix)mix=Walls[w].minx();
        if(!w || Walls[w].miny()<miy)miy=Walls[w].miny();
        if(!w || Walls[w].minz()<miz)miz=Walls[w].minz();
        if(!w || Walls[w].maxx()>max)max=Walls[w].maxx();
        if(!w || Walls[w].maxy()>may)may=Walls[w].maxy();
        if(!w || Walls[w].maxz()>maz)maz=Walls[w].maxz();
        
        unsigned ccount=0;
        if(Walls[w].p1.x == Walls[w].p2.x)ccount++;
        if(Walls[w].p1.y == Walls[w].p2.y)ccount++;
        if(Walls[w].p1.z == Walls[w].p2.z)ccount++;
        if(ccount != 1)
        {
            std::fprintf(stderr, "Wrong common count at wall %u (is %u, should be 1)\n", w, ccount);
        }
    }
    
    tree = new bsp(XYZ(mix,miy,miz), XYZ(max,may,maz), wcount);
    tree->Split(Walls);
}
#endif

const HitRec Map::FindObstacle(const XYZ &eye, const XYZ &dir)
{
    HitRec res;
    objlist_t::const_iterator i;
    FLOAT mindist = 0.0;
    for(i=objs.begin(); i!=objs.end(); ++i)
    {
        HitRec t;
        (*i)->TryHit(t, eye, dir);
        if(!t.washit)continue;
        
        const FLOAT dist = t.point.dist(eye);
        if(dist < 0.0001)continue;
        if(!res.washit || dist < mindist)
        {
            res = t;
            res.obj = *i;
            mindist = dist;
        }
    }
    return res;
}

const HitRec Map::FindObstacle(const XYZ &eye, const XYZ &dir, const Object *Except)
{
    HitRec res;
    objlist_t::const_iterator i;
    FLOAT mindist = 0.0;
    for(i=objs.begin(); i!=objs.end(); ++i)
    {
        if(*i == Except)continue;
        
        HitRec t;
        (*i)->TryHit(t, eye, dir);
        if(!t.washit)continue;
        
        const FLOAT dist = t.point.dist(eye);
        if(dist < 0.0001)continue;
        if(!res.washit || dist < mindist)
        {
            res = t;
            res.obj = *i;
            mindist = dist;
        }
    }
    return res;
}

#if 0
void Map::FindObstacleWith(HitRec &res, const XYZ &eye, const XYZ &dir, const bsp *tre)
{
    res.washit = false;
#if 1
    /* If the camera points outside, don't bother */
    if(eye.x > tre->p2.x) { if(dir.x >= 0)return; } else if(eye.x < tre->p1.x && dir.x <= 0)return;
    if(eye.y > tre->p2.y) { if(dir.y >= 0)return; } else if(eye.y < tre->p1.y && dir.y <= 0)return;
    if(eye.z > tre->p2.z) { if(dir.z >= 0)return; } else if(eye.z < tre->p1.z && dir.z <= 0)return;
#endif
    
    FLOAT mindist = 0;
    
#if 0
    bool omite = (eye.x <= tre->middle.x && dir.x < 0);
    bool omitw = (eye.x >= tre->middle.x && dir.x > 0);
    bool omitn = (eye.y <= tre->middle.y && dir.y < 0);
    bool omits = (eye.y >= tre->middle.y && dir.y > 0);
    bool omitu = (eye.z <= tre->middle.z && dir.z < 0);
    bool omitd = (eye.z >= tre->middle.z && dir.z > 0);
    
    for(unsigned a=0; a<8; ++a)
    {
        if(!tre->dirs[a])continue;
        if((a&1) ? omits : omitn)continue;
        if((a&2) ? omite : omitw)continue;
        if((a&4) ? omitd : omitu)continue;
        HitRec t = FindObstacleWith(eye, dir, tre->dirs[a]);
        if(!t.washit)continue;
        
        FLOAT dist = eye.dist(t.point);
        if(!res.washit || dist < mindist) { res = t; mindist = dist; }
    }
#endif
    
    for(unsigned a=0; a<tre->wallids.size(); ++a)
    {
        MapWall *mm = &Walls[tre->wallids[a]];
        HitRec t = mm->TryHit(eye, dir);
        if(!t.washit)continue;
        
        FLOAT dist = eye.dist(t.point);
        
        if(!res.washit || dist < mindist) { res = t; mindist = dist; }
    }
}
#endif

void Map::Add(Object *o)
{
    objs.push_back(o);
}
