#include "wall.hh"

MapWall::MapWall(const wall &w, const v3 *verts) : Pigm(w.R/255.0, w.G/255.0, w.B/255.0)
{
    const v3 &a = verts[w.v1], &b = verts[w.v2];
    p1.Set(a.x, a.y, a.z);
    p2.Set(b.x, b.y, b.z);
    
    Pigm.setdefault();
    Pigm.diffmul = w.diffper/100.0;
    Pigm.specmul = w.specper/100.0;
}

bool MapWall::hits(const XYZ &a, const XYZ &b) const
{
    return (a.x <= maxx() && b.x >= minx())
        && (a.y <= maxy() && b.y >= miny())
        && (a.z <= maxz() && b.z >= minz());
}

void MapWall::TryHit(HitRec &res, const XYZ &eye, const XYZ &dir) const
{
    res.washit = false;
    
    #define axistest(axis, x, y) \
        if(p1.axis == p2.axis) \
        { \
            res.point.axis = p1.axis; /* The common point */ \
             \
            if((eye.axis < res.point.axis && dir.axis <= 0) /* Point to wrong direction */ \
            || (eye.axis > res.point.axis && dir.axis >= 0)) return; \
             \
            res.point.x = eye.x; if(dir.axis != 0.0) res.point.x += (res.point.axis-eye.axis) * dir.x / dir.axis; \
            res.point.y = eye.y; if(dir.axis != 0.0) res.point.y += (res.point.axis-eye.axis) * dir.y / dir.axis; \
             \
            if(res.point.x < min##x() || res.point.x > max##x() \
            || res.point.y < min##y() || res.point.y > max##y()) return; \
            /* HIT */ \
            res.washit = true; \
        }
    
    axistest(x, y, z) else
    axistest(y, x, z) else
    axistest(z, x, y)
    
    #undef axistest
    
#if 0
    printf("Eye at "); eye.print();
    printf(" -> "); dir.print();
    printf("\n");
    printf("  Hits wall ");
    p1.print();
    printf(" to ");
    p2.print();
    printf("\n  At ");
    tmp.print();
    printf("\n");
#endif

    if(res.washit)
    {
        res.Pigm = Pigm;
        res.normal.Set(0,0,0);
        
        if(p1.x == p2.x)
            res.normal.x = ((p2.y > p1.y) == (p2.z > p1.z)) ? -1 : 1;
        else if(p1.y == p2.y)
            res.normal.y = ((p2.z > p1.z) == (p2.x > p1.x)) ? 1 : -1;
        else if(p1.z == p2.z)
            res.normal.z = ((p2.x > p1.x) == (p2.y > p1.y)) ? -1 : 1;
    }
}

const XYZ MapWall::Min() const
{
    return XYZ(minx(),miny(),minz());
}

const XYZ MapWall::Max() const
{
    return XYZ(maxx(),maxy(),maxz());
}
